const system = require("../system");
const redis = require("redis");
const settings = require("../../config/settings");
const bluebird = require("bluebird");
bluebird.promisifyAll(redis);
class RedisClient {
  constructor() {
    this.redisConfig = settings.redis();
    this.client = this.getCreateClientInstance();
    this.subclient = this.client.duplicate();

    this.client.on("error", function (err) {
      console.log(err, "..redisClient........error");
      // //日志记录
      // logCtl.error({
      //    optitle:"redis this.client.on异常：",
      //    op:"base/utils/redisClient/this.client.on",
      //    content:err,
      //    clientIp:""
      //  });
    });

    /**
     * 管理锁
     * @param key 换成key
     * @param expires 过期时间
     * @returns {Promise<*>}
     */
    const setLock = async (key, expires = 5) => {
      // 这里锁名修改成taskLocks
      const lock = `cron:recyclePool:taskLocks:${key}`;
      const res = await this.client.set(lock, key, 'EX', expires, 'NX');
      return res;
    };

    const delLock = async (key) => {
      const lock = `cron:recyclePool:taskLocks:${key}`;
      await this.client.del(lock);
    };
    const self = this;
    // 监听回调
    const subExpired = async (err, res) => {
      // 这里需要创建一个新的Redis对象
      // 因为 连接在订阅服务器模式下，只能使用订阅服务器命令
      const sub = self.getCreateClientInstance();
      // 设置事件参数
      const expired_subKey = `__keyevent@${self.redisConfig.db}__:expired`
      sub.subscribe(expired_subKey, function () {
        sub.on('message', async function (channel, msg) {
          try {
            var opResult = msg.split("$$");
            console.log(opResult[0], "..opResult[0]..............");
            console.log(opResult[1], "..opResult[1]..............");

            if (!opResult || opResult.length == 0) {
              return;
            }
            if (["SYTXFAIL-SYTXPUBLIC-MSGQ"].indexOf(opResult[0]) < 0) {
              return;
            }
            // 锁会在 5 秒后过期
            const lock = await setLock(opResult[1]);
            if (lock) {
              var opData = await self.zrangeList(opResult[0], opResult[1], opResult[1]);
              if (!opData || opData.length == 0) {
                delLock(opResult[1]);
                return;
              }
              var opDataItem = JSON.parse(opData[0]);
              await self.lpushData(opDataItem.queuedName, opDataItem);
              await self.zremRangebyscoreData(opResult[0], opResult[1], opResult[1]);
              delLock(opResult[1])
            }
          } catch (error) {
            //TODO：日志处理
            console.log(error.stack, ".....message......................error...........");
          }

        });
      });
    };
    // 创建监听
    this.client.send_command('config', ['set', 'notify-keyspace-events', 'Ex'], subExpired)

  }
  getDuplicateInstance() {
    return this.subclient;
  }
  getCreateClientInstance() {
    var self = this;
    return redis.createClient({
      host: self.redisConfig.host,
      port: self.redisConfig.port,
      password: self.redisConfig.password,
      db: self.redisConfig.db,
      retry_strategy: function (options) {
        if (options.total_retry_time > 1000 * 60 * 60) {
          return new Error('Retry time exhausted');
        }
        if (options.attempt > 10) {
          return 10000;
        }
        return Math.min(options.attempt * 100, 3000);
      }
    });
  }

  /**
   * 生产供消费者消费的数据
   * @param {*} queuedName 队列名称
   * @param {*} actionBody 业务数据
   */
  lpushData(queuedName, messageBody) {
    messageBody = typeof messageBody === 'object' ? JSON.stringify(messageBody) : messageBody;
    this.client.lpush(queuedName, messageBody, function (err, reply) {
      if (err) {
        console.log('lpush message error :' + err + "，queuedName：" + queuedName + "，messageBody：" + messageBody);
        return new Error('lpush message error :' + err);
      } else {
        console.log('lpush message success');
      }
    });
  }
  /**
   * 设置延迟事件redis存储
   * @param {*} key key
   * @param {*} exSeconds 过期时间-单位秒
   * @param {*} value 值
   */
  async setDelayEventData(key, value, exSeconds) {
    value = typeof value === 'object' ? JSON.stringify(value) : value;
    var key = key + "$$" + value;
    this.client.set(key, "delay event", 'EX', exSeconds);
  }

  /**
   * 添加集合数据
   * @param {*} collectionName 集合名称
   * @param {*} score 分数，用于区间查询
   * @param {*} param 参数信息（可以是json）
   */
  async zaddSortedSet(collectionName, score, param) {
    param = typeof param === 'object' ? JSON.stringify(param) : param;
    await this.client.zadd([collectionName, score, param], function (err, data) {
      if (err) {
        // console.log('zadd data error :' + err);
        return new Error('zadd data error :' + err);
      } else {
        // console.log(data, 'zadd data success');
      }
    });
  }
  /**
   * 获取集合的区间值
   * @param {*} collectionName 集合名称
   * @param {*} minValue 最小值 
   * @param {*} maxValue 最大值
   */
  async zrangeList(collectionName, minValue, maxValue) {
    return await this.client.zrangebyscoreAsync(collectionName, minValue, maxValue);
  }
  /**
   * 删除集合的区间值
   * @param {*} collectionName 集合名称
   * @param {*} minValue 最小值
   * @param {*} maxValue 最大值
   */
  async zremRangebyscoreData(collectionName, minValue, maxValue) {
    return await this.client.zremrangebyscoreAsync(collectionName, minValue, maxValue);
  }

  async setWithEx(key, val, t) {
    var p = this.client.setAsync(key, val);
    this.client.expire(key, t);
    return p;
  }
  async getCache(key) {
    return this.client.getAsync(key);
  }

}
module.exports = RedisClient;
